import sqlalchemy as sa
from ming import schema as s
from ming.odm import MappedClass
from jekyde.drivers import Driver
from jekyde.meta import JekydeModel
from sqlalchemy.orm import configure_mappers
from .conftest import BaseModel


def test_change_type(sql_session, ming_session):
    class AMingModel(MappedClass):
        class __mongometa__:
            session = ming_session
            name = "amodel"

        _id = s.ObjectId()
        value = s.String()

    class ASQLModel(BaseModel):
        __tablename__ = "amodel"

        id = sa.Column(sa.Integer(), primary_key=True)
        value = sa.Column(sa.String(length=32))

    class AModel(JekydeModel):
        _driver = Driver.Ming
        _models = {Driver.Ming: AMingModel, Driver.SQLAlchemy: ASQLModel}

    configure_mappers()

    AModel(value="test")
    ming_session.flush()
    assert AModel.query.find({}).first()

    AModel.use(Driver.SQLAlchemy)
    with sql_session() as session:
        new_obj = AModel(value="test2")
        session.add(new_obj)
        session.flush()
        created = session.query(AModel).get(new_obj.id)
        assert created


def test_migration(sql_session, ming_session):
    class BMingModel(MappedClass):
        class __mongometa__:
            session = ming_session
            name = "bmodel"

        _id = s.ObjectId()
        value = s.String()

    class BSQLModel(BaseModel):
        __tablename__ = "bmodel"

        id = sa.Column(sa.Integer(), primary_key=True)
        value = sa.Column(sa.String(length=32))

    class BModel(JekydeModel):
        _driver = Driver.Ming
        _models = {Driver.Ming: BMingModel, Driver.SQLAlchemy: BSQLModel}

    configure_mappers()
    BModel(value="test")
    ming_session.flush()
    doc = BModel.query.find({}).first()
    assert doc
    with sql_session() as session:
        new_obj = BModel.migrate_to(doc, Driver.SQLAlchemy)
        session.add(new_obj)
        session.flush()
        obj_id = new_obj.id
    with sql_session() as session:
        assert session.query(BSQLModel).get(obj_id)



import contextlib
import os

import zope.sqlalchemy
import pytest
import transaction
from ming import create_datastore
from ming.odm import ThreadLocalODMSession
from sqlalchemy import MetaData, create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker, scoped_session
from sqlalchemy.pool import NullPool


@pytest.fixture(scope="session")
def sql_session():
    engine = create_engine(
        os.getenv("DATABASE_URL"),
        poolclass=NullPool,
    )
    DBSession = scoped_session(sessionmaker())
    DBSession.configure(bind=engine)

    def get_dbsession(transaction_manager):
        zope.sqlalchemy.register(DBSession, transaction_manager=transaction_manager)
        return DBSession

    @contextlib.contextmanager
    def tm_session_ctx():
        with transaction.manager:
            yield get_dbsession(transaction.manager)

    return tm_session_ctx


@pytest.fixture(scope="session")
def ming_session():
    return ThreadLocalODMSession(
        bind=create_datastore(os.getenv("MONGODB_URL")),
    )


metadata = MetaData()
BaseModel = declarative_base(metadata=metadata)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
from __future__ import absolute_import
from __future__ import print_function

import io
import re
from glob import glob
from os.path import basename
from os.path import dirname
from os.path import join
from os.path import splitext

from setuptools import find_packages
from setuptools import setup


def read(*names, **kwargs):
    with io.open(
        join(dirname(__file__), *names), encoding=kwargs.get("encoding", "utf8")
    ) as fh:
        return fh.read()


setup(
    name="jekyde",
    version="0.0.1",
    license="MIT license",
    description="An example package. Generated with cookiecutter-pylibrary.",
    long_description="%s\n%s"
    % (
        re.compile("^.. start-badges.*^.. end-badges", re.M | re.S).sub(
            "", read("README.rst")
        ),
        re.sub(":[a-z]+:`~?(.*?)`", r"``\1``", read("CHANGELOG.rst")),
    ),
    author="Walter Danilo Galante",
    author_email="walter.galante@ovalmoney.com",
    url="https://github.com/devilicecream/jekyde",
    packages=find_packages("jekyde"),
    package_dir={"": "jekyde"},
    py_modules=[splitext(basename(path))[0] for path in glob("*.py")],
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: Unix",
        "Operating System :: POSIX",
        "Operating System :: Microsoft :: Windows",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Utilities",
    ],
    project_urls={
        "Documentation": "https://jekyde.readthedocs.io/",
        "Changelog": "https://jekyde.readthedocs.io/en/latest/changelog.html",
        "Issue Tracker": "https://github.com/devilicecream/jekyde/issues",
    },
    keywords=[
        # eg: 'keyword1', 'keyword2', 'keyword3',
    ],
    python_requires=">=3.6.*",
    install_requires=[
        "zope.sqlalchemy<=1.0",
        "six>=1.7",
        "SQLAlchemy>=1.3.0",
        "Ming<=0.9.0",
        "PyMySQL<=0.9.3",

    ],
    extras_require={},
    entry_points={},
)



from sqlalchemy.inspection import _inspects
from sqlalchemy.orm.base import _inspect_mapped_class
from sqlalchemy.schema import SchemaItem as sSchemaItem
from ming.schema import SchemaItem as mSchemaItem


class InvalidDriver(Exception):
    pass


class SameDriverException(InvalidDriver):
    pass


def jekyde_inspects(cls):
    _inspects(cls)(cls._inspect_mapped_class)
    return cls


@jekyde_inspects
class JekydeModelMeta(type):
    def __getattr__(cls, item):
        try:
            return getattr(cls._models[cls._driver], item)
        except KeyError:
            return getattr(super(JekydeModelMeta, cls), item)

    def _inspect_mapped_class(cls):
        return _inspect_mapped_class(cls._models[cls._driver])


class JekydeModel(metaclass=JekydeModelMeta):
    _models = {}
    _driver = None

    def __init__(self, *args, **kw):
        cls = self.__class__
        self._obj = cls._models[cls._driver](*args, **kw)

    def __getattr__(self, item):
        try:
            return getattr(self._obj, item)
        except KeyError:
            return getattr(super(JekydeModel, self), item)

    @classmethod
    def _check_different_driver(cls, driver):
        if driver == cls._driver:
            raise SameDriverException(f"Driver in use is already {driver}")

    @classmethod
    def use(cls, driver):
        cls._check_different_driver(driver)
        selected = cls._models.get(driver)
        if not selected:
            raise InvalidDriver(f"Can't select model for driver: {driver}")
        cls._driver = driver
        return cls

    @classmethod
    def migrate_to(cls, obj, driver):
        obj_class = obj.__class__
        values = dict(
            (name, val)
            for name, val in obj.__dict__.items()
            if isinstance(getattr(obj_class, name, None), sSchemaItem)
            or isinstance(getattr(obj_class, name, None), mSchemaItem)
        )
        new_obj = cls._models[driver](**values)
        return new_obj
